CategoryHealthChecks = "Health Checks"
CategorySecurity = "Security"
CategoryNetworking = "Networking"
CategoryResources = "Resources"
CategoryImages = "Images"
CPURequestsLabel = "CPU requests"
CPULimitsLabel = "CPU limits"
MemoryRequestsLabel = "Memory requests"
MemoryLimitsLabel = "Memory limits"
CPURequestsFailure = "CPU requests should be set"
CPULimitsFailure = "CPU limits should be set"
MemoryRequestsFailure = "Memory requests should be set"
MemoryLimitsFailure = "Memory limits should be set"
ResourceAmountTooHighFailure = "%s should be lower than %s"
ResourceAmountTooLowFailure = "%s should be higher than %s"
ResourceAmountSuccess = "%s are within the expected range"
ResourcePresentSuccess = "%s are set"
ReadinessProbeFailure = "Readiness probe should be configured"
ReadinessProbeSuccess = "Readiness probe configured"
LivenessProbeFailure = "Liveness probe should be configured"
LivenessProbeSuccess = "Liveness probe is configured"
ImageTagFailure = "Image tag should be specified"
ImageTagSuccess = "Image tag is specified"
ImagePullPolicyFailure = "Image pull policy should be \"Always\""
ImagePullPolicySuccess = "Image pull policy is \"Always\""
HostPortFailure = "Host port should not be configured"
HostPortSuccess = "Host port is not configured"
RunAsRootFailure = "Should not be allowed to run as root"
RunAsRootSuccess = "Is not allowed to run as root"
RunAsPrivilegedFailure = "Should not be running as privileged"
RunAsPrivilegedSuccess = "Not running as privileged"
ReadOnlyFilesystemSuccess = "Filesystem is read only"
ReadOnlyFilesystemFailure = "Filesystem should be read only"
PrivilegeEscalationFailure = "Privilege escalation should not be allowed"
PrivilegeEscalationSuccess = "Privilege escalation not allowed"
SecurityCapabilitiesAddedFailure = "The following security capabilities should not be added: %v"
SecurityCapabilitiesNotDroppedSuccess = "All disallowed security capabilities have been dropped"
SecurityCapabilitiesNotDroppedFailure = "The following security capabilities should be dropped: %v"
HostAliasFailure = "Host alias should not be configured"
HostAliasSuccess = "Host alias is not configured"
HostIPCFailure = "Host IPC should not be configured"
HostIPCSuccess = "Host IPC is not configured"
HostPIDFailure = "Host PID should not be configured"
HostPIDSuccess = "Host PID is not configured"
HostNetworkFailure = "Host network should not be configured"
HostNetworkSuccess = "Host network is not configured"
